@@ -1,35 +1,34 @@
 ramps.control <- function(iter = 1000, beta, sigma2.e, phi, sigma2.z, sigma2.re,
                           z.monitor = TRUE, file)
 {
-   if (!is.numeric(iter)) stop("MCMC iterations must be a numeric value or vector")
    iter <- as.integer(iter)
+   if (any(iter <= 0)) stop("only positive integers allowed for 'iter'")
    iter <- if (length(iter) == 1) 1:iter
-           else sort(unique(iter[iter > 0]))
+           else sort(unique(iter))
 
    if (missing(beta)) beta <- param(NULL)
-   else if (!is.param(beta)) stop("Incompatible data type for 'beta'")
-   else if (beta$prior != "flat")
-      stop("Only flat priors are supported for 'beta'")
+   else if (!is.param(beta)) stop("incompatible data type for 'beta'")
+   else if (beta$prior != "flat") stop("only flat priors allowed for 'beta'")
 
    if (missing(sigma2.e)) sigma2.e <- param(NULL)
-   else if (!is.param(sigma2.e)) stop("Incompatible data type for 'sigma2.e'")
+   else if (!is.param(sigma2.e)) stop("incompatible data type for 'sigma2.e'")
    else if (sigma2.e$prior != "invgamma")
-      stop("Only inverse gamma priors are supported for 'sigma2.e'")
+      stop("Only inverse gamma priors allowed for 'sigma2.e'")
 
    if (missing(phi)) phi <- param(NULL)
-   else if (!is.param(phi)) stop("Incompatible data type for 'phi'")
+   else if (!is.param(phi)) stop("incompatible data type for 'phi'")
    else if (phi$prior != "uniform")
-      stop("Only uniform priors are supported for 'sigma2.e'")
+      stop("only uniform priors allowed for 'sigma2.e'")
 
    if (missing(sigma2.z)) sigma2.z <- param(NULL)
-   else if (!is.param(sigma2.z)) stop("Incompatible data type for 'sigma2.z'")
+   else if (!is.param(sigma2.z)) stop("incompatible data type for 'sigma2.z'")
    else if (sigma2.z$prior != "invgamma")
-      stop("Only inverse gamma priors are supported for 'sigma2.z'")
+      stop("only inverse gamma priors allowed for 'sigma2.z'")
 
    if (missing(sigma2.re)) sigma2.re <- param(NULL)
-   else if (!is.param(sigma2.re)) stop("Incompatible data type for 'sigma2.re'")
+   else if (!is.param(sigma2.re)) stop("incompatible data type for 'sigma2.re'")
    else if (sigma2.re$prior != "invgamma")
-      stop("Only inverse gamma priors are supported for 'sigma2.re'")
+      stop("only inverse gamma priors allowed for 'sigma2.re'")
 
    fnames <- list(params = NULL, z = NULL)
    if (!missing(file)) {
@@ -47,12 +46,12 @@ ramps.control <- function(iter = 1000, beta, sigma2.e, phi, sigma2.z, sigma2.re,
 
    list(beta = beta, sigma2.e = sigma2.e, phi = phi, sigma2.z = sigma2.z,
         sigma2.re = sigma2.re, z = list(monitor = z.monitor), iter = iter,
-        file = fnames)
+        file = fnames, expand = 0)
 }
 
 
 param <- function(init, prior = c("flat", "invgamma", "normal", "uniform"),
-                  tuning = 1, monitor = TRUE, ...)
+                  tuning = 1, ...)
 {
    retval <- list(init = NULL)
    retval$prior <- match.arg(prior)
@@ -64,20 +63,20 @@ param <- function(init, prior = c("flat", "invgamma", "normal", "uniform"),
       },
       invgamma = {
          if (!is.numeric(hyper$shape) || hyper$shape <= 0)
-            stop("Inverse gamma shape hyperparameter must be numeric > 0")
+            stop("inverse gamma shape hyperparameter must be numeric > 0")
          if (!is.numeric(hyper$scale) || hyper$scale <= 0)
-            stop("Inverse gamma scale hyperparameter must be numeric > 0")
+            stop("inverse gamma scale hyperparameter must be numeric > 0")
          retval$shape <- rep(hyper$shape, length.out = n)
          retval$scale <- rep(hyper$scale, length.out = n)
          if (sum(na <- is.na(init)) > 0)
             init[na] <- 1 / rgamma(sum(na), retval$shape[na] / retval$scale[na], 1)
-         if (any(init <= 0)) stop("Initial values must be greater than 0")
+         if (any(init <= 0)) stop("initial values must be greater than 0")
       },
       normal = {
          if (!is.numeric(hyper$mean))
-            stop("Normal mean hyperparameter must be numeric")
+            stop("normal mean hyperparameter must be numeric")
          if (!is.numeric(hyper$var))
-            stop("Normal variance hyperparameter must be numeric")
+            stop("normal variance hyperparameter must be numeric")
          retval$mean <- rep(hyper$mean, length.out = n)
          if (is.matrix(hyper$var)) {
             val <- hyper$var
@@ -85,38 +84,36 @@ param <- function(init, prior = c("flat", "invgamma", "normal", "uniform"),
             val <- diag(n)
             diag(val) <- rep(hyper$var, length.out = n)
          }
-         if (any(dim(val) != n)) stop("Non-comformable normal variance matrix")
+         if (any(dim(val) != n)) stop("non-comformable normal variance matrix")
          val <- try(chol(val))
          if (class(val) == "try-error")
-            stop("Normal variance hyperparameter must be positive definate")
+            stop("normal variance hyperparameter must be positive definate")
          retval$precision <- chol2inv(val)
          if (sum(na <- is.na(init)) > 0)
             init[na] <- (retval$mean + val %*% rnorm(n))[na]
       },
       uniform = {
          if (!is.numeric(hyper$min))
-            stop("Uniform min hyperparameter must be numeric")
+            stop("uniform min hyperparameter must be numeric")
          if (!is.numeric(hyper$max))
-            stop("Uniform max hyperparameter must be numeric")
+            stop("uniform max hyperparameter must be numeric")
          if (any(hyper$min >= hyper$max))
-            stop("Uniform min hyperparameter must be < max")
+            stop("uniform min hyperparameter must be < max")
          retval$min <- rep(hyper$min, length.out = n)
          retval$max <- rep(hyper$max, length.out = n)
          if (sum(na <- is.na(init)) > 0)
             init[na] <- runif(sum(na), retval$min[na], retval$max[na])
          if (any(init <= retval$min, init >= retval$max))
-            stop("Initial values must be contained in (min, max)")
+            stop("initial values must be contained in (min, max)")
       }
    )
 
-   if (n > 0 && !is.numeric(init)) stop("Initial values must be numeric")
+   if (n > 0 && !is.numeric(init)) stop("initial values must be numeric")
    retval$init <- init
 
-   if (any(tuning <= 0)) stop("Tuninig parameters must be > 0")
+   if (any(tuning <= 0)) stop("tuninig parameters must be > 0")
    retval$tuning <- rep(tuning, length.out = n)
 
-   retval$monitor <- as.logical(rep(monitor, length.out = n))
-
    structure(retval, class = "param")
 }
 
